 /*
 **
 ** File:         CFRException.m
 **
 ** Created by:   bbum
 **
 **
 ** Standard disclaimer here.
 **
 ** This code may not be correct.  It may not even work [but it sure seems to].
 ** Potentially,  it may corrupt your data irretrievably [if it does,
 ** please let us know-- we will do our best to make sure the same thing
 ** doesn't happen to us or anyone else].  
 */

// import APIs to NeXT provided frameworks
#import <Foundation/Foundation.h>

// import local framework's API
#import "CodeFabSupport.h"

// import API required througout this file's scope
#import "CFRExceptionTracer.h"

/*"
 * Exception raised when one of the assertion macros like CFRValidateMethodArgument() fails.
"*/
NSString *CFRInvalidVariableException = @"CodeFabRuntime: Invalid Variable Value Exception.";

@implementation CFRExceptionTracer
/*"
 * A useful debugging tool that provides exception tracing.   When the exception tracer is installed and enabled, a message will
 * be logged on the console whenever the #{-raise} method of #NSException is about to be invoked.  As such, this class allows the
 * developer to also monitor exceptions that are handled within an NS_HANDLER block.
"*/

static BOOL exceptionTracingInstalled = NO;
static BOOL exceptionTracingEnabled = NO;

+ (void) testExceptionTracing
    /*"
     * Tests the exception tracing functionality provided by this class.  Of course, this method invokes
     * #{+installExceptionTracing} and, as such, will modify the runtime in a manner that cannot be undone.
    "*/
{
    NSAutoreleasePool *pool;
    id randomObject;

    pool = [[NSAutoreleasePool alloc] init];
    randomObject = [[NSObject alloc] init];

    [self installExceptionTracing];

    NS_DURING
        [self logMethod: _cmd withFormat: @"Raising...."];
        [(NSArray *) randomObject objectAtIndex: 0];
        [self logMethod: _cmd withFormat: @"SHOULDN'T HAVE HAPPENED (1)"];
    NS_HANDLER
        [self logMethod: _cmd withFormat: @"Handled."];
        [pool release];
        NS_VOIDRETURN;
    NS_ENDHANDLER
    [self logMethod: _cmd withFormat: @"HOULDN'T HAVE HAPPENED (2)"];
    [pool release];
}

+ (void) installExceptionTracing
    /*"
     * Installs the exception tracing mechanism.   Once installed, the tracer cannot be uninstalled.   The installation is
     * performed by telling the #CFRExceptionTracer class to pose as the NSException class.
     * 
     * Internally, this method creates and destroys an instance of NSAutoreleasePool around the installation of the tracer.  As
     * such, this method can be invoked at any time during application start up.
     * 
     * Upon successful invocation, the exception tracer will be installed and enabled.
    "*/
{
    NSAutoreleasePool *pool;

    if (exceptionTracingInstalled)
        return;

    pool = [[NSAutoreleasePool alloc] init];
    [self logMethod: _cmd withFormat: @"Installing exception tracing..."];
    [CFRExceptionTracer class]; // ensure classes are intialized
    [CFRExceptionTracer poseAsClass: [NSException class]];
    [pool release];

    exceptionTracingInstalled =YES;
    exceptionTracingEnabled = YES;
}

+ (void) setExceptionTracingEnabled: (BOOL) aFlag
    /*"
     * Enables and disables the exception tracer.  Enabling the tracer does not automatically install it.
    "*/
{
    exceptionTracingEnabled = aFlag;
}

+ (BOOL) exceptionTracingEnabled
    /*"
     * Returns YES if the exception tracer is enabled (does not test to see if it is installed).
    "*/
{
    return exceptionTracingEnabled;
}

+ (BOOL) exceptionTracingInstalled
    /*"
     * Returns YES if the exception tracing mechanism has been installed.
    "*/
{
    return exceptionTracingInstalled;
}

- (volatile void)raise
    /*"
     * Upon installation, the exception tracer class #CFRExceptionTracer poses as #NSException.  As such, this method overrides
     * #NSException's implementation of #{-raise}.   If exception tracing is enabled, this method logs a nicely formatted message
     * via #{-logMethod:withFormat:} indicating the name, reason, and userInfo for the exception that is about to be raised.
     * 
     * Finally, this method invokes #NSException's original implementation of #{-raise} to cause the normal exception behaviour to
     * occur.
    "*/
{
    if (exceptionTracingEnabled)
        [self logMethod: _cmd
             withFormat: @"Will raise exception:\n\tName: %@\n\tReason: %@\n\tuserInfo: %@\n",
            [self name], [self reason], [self userInfo]];
    [super raise];
}
@end
